import React, { useState } from 'react';
import { View, Text, StyleSheet, Modal, TouchableOpacity, TextInput, Platform } from 'react-native';
import { Picker } from '@react-native-picker/picker';
import DefaultButton from "../buttons/DefaultButton";

interface Props {
    options: string[];
    title: string;
    selectOption: (option: string) => void;
    type: number;
}

export default function ComboBox({ options, title, selectOption, type }: Props) {
    const [selectedOption, setSelectedOption] = useState(options[0]);
    const [comboBoxIsVisible, setComboBoxIsVisible] = useState(false);

    const handleOpen = () => {
        setComboBoxIsVisible(true);
    };

    const handleClose = () => {
        setComboBoxIsVisible(false);
    };

    return (
        <View style={styles.container}>
            <DefaultButton text={title} handleButtonPressed={handleOpen} buttonStyle={styles.buttonStyle} />
            <View style={styles.textContainer}>
                <Text style={styles.textOption}>{selectedOption}</Text>
            </View>
            <Modal visible={comboBoxIsVisible} transparent={true} animationType="slide">
                <View style={styles.modalOverlay}>
                    <View
                        style={[
                            styles.modalContent,
                            type === 0 && Platform.OS === 'ios' ? { height: 350 } : { height: 150 },
                        ]}
                    >
                        {type === 0 ? (
                            <Picker
                                selectedValue={selectedOption}
                                onValueChange={(itemValue) => {
                                    setSelectedOption(itemValue);
                                    selectOption(itemValue);
                                }}
                                style={styles.picker}
                            >
                                {options.map((option, index) => (
                                    <Picker.Item key={index} label={option} value={option} />
                                ))}
                            </Picker>
                        ) : (
                            <TextInput
                                onChangeText={(value) => {
                                    let numericValue = parseInt(value);
                                    if (!isNaN(numericValue)) {
                                        if (numericValue < parseInt(options[0])) numericValue = parseInt(options[0]);
                                        if (numericValue > parseInt(options[1])) numericValue = parseInt(options[1]);

                                        selectOption(numericValue.toString());
                                        setSelectedOption(numericValue.toString());
                                    } else {
                                        setSelectedOption('');
                                    }
                                }}
                                keyboardType="numeric"
                                value={selectedOption}
                                style={styles.textInput}
                            />
                        )}
                        <TouchableOpacity onPress={handleClose} style={styles.closeButton}>
                            <Text style={styles.closeButtonText}>Close</Text>
                        </TouchableOpacity>
                    </View>
                </View>
            </Modal>
        </View>
    );
}

const styles = StyleSheet.create({
    container: {
        flex: 1,
        justifyContent: 'center',
        alignItems: 'center',
        width: '100%',
    },
    textContainer: {
        width: '100%',
        justifyContent: 'center',
        alignItems: 'center',
        marginTop: 20,
    },
    modalOverlay: {
        flex: 1,
        justifyContent: 'center',
        alignItems: 'center',
    },
    modalContent: {
        width: '80%',
        backgroundColor: '#fff',
        borderRadius: 10,
        padding: 20,
        alignItems: 'center',
        borderWidth: 2,
        borderColor: 'black',
        borderStyle: 'solid',
    },
    picker: {
        width: '100%',
    },
    closeButton: {
        marginTop: 20,
        padding: 10,
        backgroundColor: '#7988d7',
        borderRadius: 5,
    },
    closeButtonText: {
        color: '#fff',
        fontWeight: 'bold',
    },
    textOption: {
        fontSize: 20,
        fontWeight: 'bold',
    },
    buttonStyle: {
        backgroundColor: '#7988D7',
    },
    textInput: {
        borderWidth: 2,
        borderColor: 'black',
        borderStyle: 'solid',
        width: '100%',
        padding: '2%',
    },
});
