import {QuizInformations} from "../../../../models/QuizInformations";
import {Question} from "../../../../models/Question";
import {NavigationProp} from "@react-navigation/native";
import React, {useEffect, useState} from "react";
import {useQuizService} from "../../../../services/QuizService";
import {Answer} from "../../../../models/Answer";
import HttpError from "../../../../services/HttpError";
import AnswerButton from "../../../../components/buttons/AnswerButton";
import {ActivityIndicator, View, Text, StyleSheet} from "react-native";
import BlueButton from "../../../../components/buttons/BlueButton";
import {ButtonsStyles} from "../../../../styles/ButtonsStyles";
import {QuestionResponse} from "../../../../models/Response/QuestionResponse";
import {RunInformations} from "../../../../models/RunInformations";
import {Socket} from "socket.io-client";
import {AnswerWsResponse} from "../../../../models/Response/AnswerWsResponse";


interface Props {
    runId?: string;
    actualQuestion?: QuestionResponse;
    fetchActualQuestion: () => void;
    socket: Socket;
    roomId: string;
    navigation: NavigationProp<any>;
}

enum QuizState {
    ANSWERING= "Answering",
    LOADING= "Loading",
    SHOWING_RESULTS= "ShowingResults",
}

const getStyleOfAnswerButtonText = (answerId: number, selectedAnswerIndex: number | null, correctAnswerId: number | null, quizState: QuizState) => {
    if(quizState === QuizState.LOADING)
    {
        return styles.loadingText;
    }
    if(quizState === QuizState.ANSWERING)
    {
        return styles.answerText;

    }
    if(quizState === QuizState.SHOWING_RESULTS)
    {
        if(answerId === correctAnswerId) return styles.correctText;
        if(answerId !== correctAnswerId && selectedAnswerIndex === answerId) return styles.incorrectText;
    }
    return styles.answerText;
}

const getStyleOfAnswerButton = (answerId: number, selectedAnswerIndex: number | null, correctAnswerId: number | null, quizState: QuizState) => {
    if(quizState === QuizState.LOADING)
    {
        return styles.loadingAnswer;
    }
    if(quizState === QuizState.ANSWERING)
    {
        if(selectedAnswerIndex === null) return styles.answerButton;
        if(selectedAnswerIndex === answerId) return styles.selectedAnswer;

    }
    if(quizState === QuizState.SHOWING_RESULTS)
    {
        if(answerId === correctAnswerId) return styles.correctAnswer;
        if(answerId !== correctAnswerId && selectedAnswerIndex === answerId) return styles.incorrectAnswer;
    }
    return styles.answerButton;
}

export default function PlayingQuizMultiModeBody({runId, actualQuestion, fetchActualQuestion, roomId, socket, navigation }: Props) {
    const [selectedAnswerId, setSelectedAnswerId] = useState<number | null>(null);
    const [quizState, setQuizState] = useState(QuizState.ANSWERING);
    const [correctAnswerId, setCorrectAnswerId] = useState<number | null>(null);
    const [isLastQuestion, setIsLastQuestion] = useState(false);
    const [quizInformations, setQuizInformations] = useState<QuizInformations>();
    const [runInformations, setRunInformations] = useState<RunInformations>();
    const {answerQuestion, getRunsInfo, getQuizInformations} = useQuizService();
    const getCorrectAnswer = (answers: Answer[]): Answer => {
        const correctAnswer = answers.find((answer) => answer.isCorrect);
        if (!correctAnswer) return answers[0];
        return correctAnswer;
    };

    const initSocket = async () => {
        console.log("init socket in body");
        socket.on("answersRevealed", (data: AnswerWsResponse) => {
            const correctAnswerIdFetched = getCorrectAnswer(data.answers).id;
            console.log("correctAnswerIdFetched", correctAnswerIdFetched);
            setCorrectAnswerId(correctAnswerIdFetched);
            setQuizState(QuizState.SHOWING_RESULTS);
        });
        // socket.on("score", (data) => {
        //     console.log("Score:", data);
        // });
    }

    useEffect(() => {
        initSocket();
    }, []);



    const onValidation = async () => {

        if(selectedAnswerId === null || actualQuestion === undefined) return;
        setQuizState(QuizState.LOADING);
        socket.emit("submitAnswer", {
            roomId,
            runId,
            questionId: actualQuestion.question.id,
            answerId: selectedAnswerId,
        });

        setTimeout(() => {
            socket.emit("revealAnswer", {runId: runId});
        },500);
    };
    const onContinue = () => {
        socket.emit("nextQuestion");
        setQuizState(QuizState.ANSWERING);
        setCorrectAnswerId(null);
        setSelectedAnswerId(null);
    }

    const onAnsweredButtonClicked = (answerId: number) => {
        if(quizState === QuizState.SHOWING_RESULTS || quizState === QuizState.LOADING) {
            return;
        }
        setSelectedAnswerId(answerId);
    }

    return (
        <View style={styles.buttonContainer}>
            {actualQuestion ? (
                <>
                    {actualQuestion.question.answers.map((answer, index) => (
                        <AnswerButton
                            key={index}
                            text={answer.text}
                            handleButtonPressed={() => onAnsweredButtonClicked(answer.id)}
                            buttonStyle={getStyleOfAnswerButton(answer.id, selectedAnswerId, correctAnswerId, quizState)}
                            buttonText={getStyleOfAnswerButtonText(answer.id, selectedAnswerId, correctAnswerId, quizState)}
                        />
                    ))}
                    <View style={styles.playButtonContainer}>
                        <BlueButton onPress={() => quizState === QuizState.ANSWERING ? onValidation() : onContinue()} text={quizState === QuizState.ANSWERING ? "CONFIRM" : "CONTINUE"} buttonStyle={{borderRadius: 50}} isDisabled={false}/>
                    </View>
                </>
            ) : (
                <View style={styles.loadingContainer}>
                    <ActivityIndicator size="large" color="#2b73fe" />
                    <Text style={styles.loadingText}>Loading...</Text>
                </View>
            )}
        </View>
    );

}

const styles = StyleSheet.create({
    buttonContainer: {
        display: 'flex',
        flex: 1,
        alignItems: 'center',
        justifyContent: 'flex-start',
        width: '100%',
        marginVertical: '3%',
    },
    loadingContainer: {
        flex: 1,
        justifyContent: "center",
        alignItems: "center",
    },
    playButtonContainer: {
        marginTop: 20, // Ajoute un espacement au-dessus du bouton Play
        width: '80%', // S'assure que le bouton prend toute la largeur disponible
        alignItems: 'center', // Centre le bouton horizontalement
    },
    correctAnswer: {
        ...ButtonsStyles.answerButton,
        borderColor: 'green',
        borderWidth: 2,
    },
    incorrectAnswer: {
        ...ButtonsStyles.answerButton,
        borderColor: 'red',
        borderWidth: 2,
    },
    loadingAnswer: {
        ...ButtonsStyles.answerButton,
        backgroundColor: '#4F6367',
        borderWidth: 2,
    },
    selectedAnswer: {
        ...ButtonsStyles.answerButton,
        borderColor: 'grey',
        borderWidth: 2,
    },
    answerText: {
        color: 'black',
        textAlign: 'center',
    },
    correctText: {
        fontWeight: 'bold',
        color: 'green',
    },
    incorrectText: {
        fontWeight: 'bold',
        color: 'red',
    },
    loadingText: {
        color: 'black',
        textAlign: 'center',
    },
    answerButton: {
        backgroundColor: "#F3F3F3",
        height: '15%',
        alignItems: 'center',
        width: '80%',
        marginVertical: '3%',
        justifyContent: "center",
        shadowColor: "#000",
        shadowOffset: { width: 0, height: 2 },
        shadowOpacity: 0.25,
        shadowRadius: 3.5,
        elevation: 5,
    },
});

