import HttpError from "./HttpError";
import {QuizCommunityResponse} from "../models/Quiz";
import axios from "axios";
import {Answer} from "../models/Answer";
import {Question} from "../models/Question";
import {AnswerResponse} from "../models/Response/AnswerResponse";
import {QuizInformations} from "../models/QuizInformations";
import {QuestionResponse} from "../models/Response/QuestionResponse";
import {QuizGenerateAnswer} from "../models/QuizGenerateAnswer";
import {RunInformations} from "../models/RunInformations";
import {AllRunsResponse} from "../models/Response/AllRunsResponse";

export const useQuizService =  () => {
    const url = "https://klebert-host.com:33037"; // Remplace par l'URL de ton endpoint API

    const generateQuiz = async (amount: number, category: number, difficulty: string) : Promise<QuizGenerateAnswer | HttpError> => {
        const requestBody = {
            amount,
            category,
            difficulty: difficulty.toLowerCase(),
        };

        try {
            const response = await axios.post<QuizGenerateAnswer>(`${url}/quiz/generate`, requestBody, {
                headers: {
                    "Content-Type": "application/json",
                },
            });
            const runId =  response.data.id;
            return {id: runId, quizId: response.data.quizId};
        } catch (error: any) {
            console.log("Error while generating quiz:", error);

            if (error.response) {
                // Gestion des erreurs HTTP
                return new HttpError(error.response.status, error.response.data?.message || "HTTP error");
            } else {
                // Gestion des erreurs réseau ou autres
                return new HttpError(500, "Unexpected error: " + error.message);
            }
        }
    }

    const getActualQuestionOfAQuiz = async (quizId: string): Promise<Question | HttpError> => {
        const requestBody = {
            id: quizId,
        };

        try {
            const response = await axios.post<QuestionResponse>(`${url}/run/question`, requestBody, {
                headers: {
                    "Content-Type": "application/json",
                },
            });

            return response.data.question;
        } catch (error: any) {
            console.log("Error while remaining quiz:", error);

            if (error.response) {
                // Gestion des erreurs HTTP
                return new HttpError(error.response.status, error.response.data?.message || "HTTP error");
            } else {
                // Gestion des erreurs réseau ou autres
                return new HttpError(500, "Unexpected error: " + error.message);
            }
        }
    }

    const getQuizInformations = async (quizId: string): Promise<QuizInformations | HttpError> => {

        try {
            const response = await axios.get<QuizInformations>(`${url}/quiz/info/`+quizId, {
                headers: {
                    "Content-Type": "application/json",
                },
            });
            return response.data;
        } catch (error: any) {
            console.log("Error while getQuizInformations quiz:", error);

            if (error.response) {
                // Gestion des erreurs HTTP
                return new HttpError(error.response.status, error.response.data?.message || "HTTP error");
            } else {
                // Gestion des erreurs réseau ou autres
                return new HttpError(500, "Unexpected error: " + error.message);
            }
        }
    }

    const answerQuestion = async (quizId: string, questionId: number, answerId: number): Promise<AnswerResponse | HttpError> => {
        const requestBody = {
            id: quizId,
            questionID: questionId,
            answerID: answerId,
        };

        try {
            const response = await axios.post<AnswerResponse>(`${url}/run/answer`, requestBody, {
                headers: {
                    "Content-Type": "application/json",
                },
            });

            return response.data;
        } catch (error: any) {
            console.log("Error while remaining quiz:", error);

            if (error.response) {
                // Gestion des erreurs HTTP
                return new HttpError(error.response.status, error.response.data?.message || "HTTP error");
            } else {
                // Gestion des erreurs réseau ou autres
                return new HttpError(500, "Unexpected error: " + error.message);
            }
        }
    };

    // const remainingQuiz = async (id: string): Promise<Quiz | HttpError> => {
    //     const requestBody = {
    //         id,
    //     };
    //
    //     try {
    //         const response = await axios.post<Quiz>(`${url}/quiz/remaining`, requestBody, {
    //             headers: {
    //                 "Content-Type": "application/json",
    //             },
    //         });
    //
    //         // Récupération des données retournées par l'API
    //         return shuffleAnswersOfQuiz(response.data);
    //     } catch (error: any) {
    //         console.log("Error while remaining quiz:", error);
    //
    //         if (error.response) {
    //             // Gestion des erreurs HTTP
    //             return new HttpError(error.response.status, error.response.data?.message || "HTTP error");
    //         } else {
    //             // Gestion des erreurs réseau ou autres
    //             return new HttpError(500, "Unexpected error: " + error.message);
    //         }
    //     }
    // };

    const getAnswerQuiz = async (
        codeQuiz: string,
        questionID: number,
        answerID: number
    ): Promise<Answer[] | HttpError> => {
        try {
            const requestBody = {
                id: codeQuiz,
                questionID: questionID,
                answerID: answerID,
            };

            const response = await axios.post(`${url}/quiz/answer`, requestBody, {
                headers: {
                    "Content-Type": "application/json",
                },
            });

            const answers: Answer[] = response.data.answers.map((answer: any) => ({
                id: answer.id,
                text: answer.text,
                isCorrect: answer.isCorrect,
                questionId: answer.questionId,
            }));

            return answers;
        } catch (error: any) {
            console.log("Error while fetching answers:", error);

            if (error.response) {
                // Gestion des erreurs HTTP
                return new HttpError(error.response.status, error.response.data?.message || "HTTP error");
            } else {
                // Gestion des erreurs réseau ou autres
                return new HttpError(500, "Unexpected error: " + error.message);
            }
        }
    };

    const getRandomQuiz = async (): Promise<QuizGenerateAnswer | HttpError> => {
        const categories = await getCategories();
        const difficulties = ['easy', 'medium', 'hard'];
        const requestBody = {
            amount: 10,
            category: categories[Math.floor(Math.random() * categories.length)].id,
            difficulty: difficulties[Math.floor(Math.random() * difficulties.length)],
        };

        try {
            const response = await axios.post<QuizGenerateAnswer>(`${url}/quiz/generate`, requestBody, {
                headers: {
                    "Content-Type": "application/json",
                },
            });
            const runId =  response.data.id;
            // Récupération du run ID
            return {id: runId, quizId: response.data.quizId};
        } catch (error: any) {
            console.log("Error while generating random quiz:", error);

            if (error.response) {
                // Gestion des erreurs HTTP
                return new HttpError(error.response.status, error.response.data?.message || "HTTP error");
            } else {
                // Gestion des erreurs réseau ou autres
                return new HttpError(500, "Unexpected error: " + error.message);
            }
        }
    };

    const getCommunityQuiz = async (skip: number, take: number) => {
        try {
            const response = await axios.get<QuizCommunityResponse>(`${url}/quiz/paginated?skip=${skip}&take=${take}`, {
                withCredentials: true, // Inclut les cookies pour une session authentifiée
            });

            return response.data.quizzes;
        } catch (error: any) {

            if (error.response) {
                return new HttpError(error.response.status, error.response.data?.message || "HTTP error");
            } else {
                return new HttpError(500, "Unexpected error: " + error.message);
            }
        }
    };

    const restartQuiz = async (id: string) : Promise<boolean | HttpError> => {
        try {
            const requestBody = {
                id
            };

            await axios.patch(`${url}/quiz/restart`, requestBody, {
                withCredentials: true,
            });

            return true;
        } catch (error: any) {
            if (error.response) {
                return new HttpError(error.response.status, error.response.data?.message);
            } else {
                return new HttpError(500, "Unexpected error: " + error.message);
            }
        }
    }

    const getCategories = async () => {
        try {
            const response = await axios.get(`${url}/quiz/categories`, {
                headers: {
                    "Content-Type": "application/json",
                },
            });
            return response.data;
        } catch (error: any) {
            if (error.response) {
                return new HttpError(error.response.status, error.response.data?.message || "HTTP error");
            } else {
                return new HttpError(500, "Unexpected error: " + error.message);
            }
        }
    };

    const getRunsInfo = async (id: string): Promise<RunInformations | HttpError> => {
        try {
            const response = await axios.get<RunInformations>(`${url}/run/info/`+id, {
                headers: {
                    "Content-Type": "application/json",
                },
            });
            return response.data;
        } catch (error: any) {
            console.log("Error while getQuizInformations quiz:", error);

            if (error.response) {
                return new HttpError(error.response.status, error.response.data?.message || "HTTP error");
            } else {
                return new HttpError(500, "Unexpected error: " + error.message);
            }
        }
    }
    
    const getAllRuns = async (): Promise<AllRunsResponse | HttpError> => {
        try {
            const response = await axios.get<AllRunsResponse>(`${url}/run/userRuns`, {
                headers: {
                    "Content-Type": "application/json",
                },
            });

            return response.data;
        } catch (error: any) {
            console.log("Error while getAllRuns:", error);
            if (error.response) {
                return new HttpError(error.response.status, error.response.data?.message || "HTTP error");
            } else {
                return new HttpError(500, "Unexpected error: " + error.message);
            }
        }
    }

     const createRun = async (quizId: string): Promise<string | HttpError> => {
        try {
            const response = await axios.post(`${url}/run/start`, {
                quizId: quizId,
            }, {
                headers: {
                    "Content-Type": "application/json",
                },
            });

            return response.data.id;
        } catch (error: any) {
            console.log("Error while getQuizInformations quiz:", error);

            if (error.response) {
                return new HttpError(error.response.status, error.response.data?.message || "HTTP error");
            } else {
                return new HttpError(500, "Unexpected error: " + error.message);
            }
        }
    }

    return {
        generateQuiz: generateQuiz,
        getAnswerQuiz: getAnswerQuiz,
        // remainingQuiz: remainingQuiz,
        getRandomQuiz: getRandomQuiz,
        getCommunityQuiz: getCommunityQuiz,
        restartQuiz: restartQuiz,
        getCategories: getCategories,
        answerQuestion: answerQuestion,
        getActualQuestionOfAQuiz: getActualQuestionOfAQuiz,
        getQuizInformations: getQuizInformations,
        getRunsInfo: getRunsInfo,
        getAllRuns: getAllRuns,
        createRun: createRun,
    }
}